@@ -11,14 +11,15 @@ from ast import parse
 from ast import Assign
 from ast import Global
 from ast import Attribute
+from ast import Subscript
 from ast import FunctionDef
 from ast import NodeVisitor
 
 from .utils import YieldSearch
 
 
-ClassDef = namedtuple('ClassDef', 'name')
-FunctionDef = namedtuple('FunctionDef', 'name')
+ClassDefNode = namedtuple('ClassDef', 'name')
+FunctionDefNode = namedtuple('FunctionDef', 'name')
 
 
 class Writer:
@@ -56,7 +57,7 @@ class Pythonium(NodeVisitor):
 
     def visit(self, node):
         if os.environ.get('DEBUG', False):
-            print(">>>", node.__class__.__name__, node._fields)
+            sys.stderr.write(">>> {} {}\n".format(node.__class__.__name__, node._fields))
         return super().visit(node)
 
     def visit_Pass(self, node):
@@ -68,11 +69,7 @@ class Pythonium(NodeVisitor):
         list(map(self.visit, node.body))
         self.writer.pull()
         self.writer.write('}')
-        self.writer.write('catch(__exception__) {')
-        self.writer.push()
         list(map(self.visit, node.handlers))
-        self.writer.pull()
-        self.writer.write('}')
 
     def visit_Raise(self, node):
         
@@ -81,29 +78,42 @@ class Pythonium(NodeVisitor):
     def visit_ExceptHandler(self, node):
         # 'type', 'name', 'body'
         if node.type:
-            self.writer.write('if (pythonium_is_exception(__exception__, {}) {{'.format(self.visit(node.type)))
-            self.writer.push()
-        if node.name:
-            self.writer.write('var {} = __exception__'.format(node.name))
+            if node.name:
+                catch = '{} if (pythonium_is_exception({}, {}))'.format(node.name, node.name, self.visit(node.type))
+            else:
+                catch = '__exception__ if (pythonium_is_exception(__exception__, {}))'.format(self.visit(node.type))
+        elif node.name:
+            catch = node.name
+        else:
+            catch = '__exception__'
+        self.writer.write('catch ({}) {{'.format(catch))
+        self.writer.push()
         list(map(self.visit, node.body))
-        if node.type:
-            self.writer.pull()
-            self.writer.write('}')
+        self.writer.pull()
+        self.writer.write('}')
 
     def visit_Yield(self, node):
         return 'yield {}'.format(self.visit(node.value))
         
     def visit_In(self, node):
-        return ' in '
+        return '__in__'
 
     def visit_Module(self, node):
         list(map(self.visit, node.body))
 
     def visit_Tuple(self, node):
-        return '[{}]'.format(', '.join(map(self.visit, node.elts)))
+        args = ', '.join(map(self.visit, node.elts))
+        if args:
+            return 'pythonium_call(list, [{}])'.format(args)
+        else:
+            return 'pythonium_call(list)'
 
     def visit_List(self, node):
-        return '[{}]'.format(', '.join(map(self.visit, node.elts)))
+        args = ', '.join(map(self.visit, node.elts))
+        if args:
+            return 'pythonium_call(list, [{}])'.format(args)
+        else:
+            return 'pythonium_call(list)'
 
     def visit_ImportFrom(self, node):
         if len(node.names) > 1:
@@ -135,17 +145,17 @@ class Pythonium(NodeVisitor):
 
     def _is_inside_method_definition(self):
         if len(self._def_stack) >= 2:
-            if isinstance(self._def_stack[-2], ClassDef):
+            if isinstance(self._def_stack[-2], ClassDefNode):
                 if isinstance(self._def_stack[-1], FunctionDef):
                     return True
         return False
 
     def _is_inside_class_definition(self):
-        return isinstance(self._def_stack[-1], ClassDef)
+        return isinstance(self._def_stack[-1], ClassDefNode)
 
     def visit_FunctionDef(self, node):
         # 'name', 'args', 'body', 'decorator_list', 'returns'
-        self._def_stack.append(FunctionDef(node.name))
+        self._def_stack.append(FunctionDefNode(node.name))
         args, kwargs, varargs, varkwargs = self.visit(node.args)
         
         all_parameters = list(args)
@@ -238,7 +248,7 @@ class Pythonium(NodeVisitor):
         return self.visit(node.value)
 
     def visit_Subscript(self, node):
-        return "call({}, '__getitem__')({})".format(self.visit(node.value), self.visit(node.slice))
+        return "pythonium_call(pythonium_get_attribute({}, '__getitem__'), {})".format(self.visit(node.value), self.visit(node.slice))
 
     def visit_arguments(self, node):
         # 'args', 'vararg', 'varargannotation', 'kwonlyargs', 'kwarg', 'kwargannotation', 'defaults', 'kw_defaults'
@@ -252,11 +262,11 @@ class Pythonium(NodeVisitor):
 
     def visit_Name(self, node):
         if node.id == 'None':
-            return 'undefined'
+            return '__NONE'
         elif node.id == 'True':
-            return 'true'
+            return '__TRUE'
         elif node.id == 'False':
-            return 'false'
+            return '__FALSE'
         elif node.id == 'null':
             return 'null'
         return node.id.replace('__DOLLAR__', '$')
@@ -357,7 +367,7 @@ class Pythonium(NodeVisitor):
             self.writer.write('if(!{}) {{ continue; }}'.format(' && '.join(map(self.visit, node.ifs))))
 
     def visit_While(self, node):
-        self.writer.write('while({}) {{'.format(self.visit(node.test)))
+        self.writer.write('while(pythonium_is_true({})) {{'.format(self.visit(node.test)))
         self.writer.push()
         list(map(self.visit, node.body))
         self.writer.pull()
@@ -365,7 +375,7 @@ class Pythonium(NodeVisitor):
 
     def visit_AugAssign(self, node):
         target = self.visit(node.target)
-        self.writer.write('{} = {}.{}({});'.format(target, target, self.visit(node.op), self.visit(node.value)))
+        self.writer.write('{} = pythonium_call(pythonium_get_attribute({}, "{}"), {});'.format(target, target, self.visit(node.op), self.visit(node.value)))
 
     def visit_Str(self, node):
         s = node.s.replace('\n', '\\n')
@@ -377,7 +387,7 @@ class Pythonium(NodeVisitor):
         left = self.visit(node.left)
         op = self.visit(node.op)
         right = self.visit(node.right)
-        return '({}.{}({}))'.format(left, op, right)
+        return '(pythonium_call(pythonium_get_attribute({}, "{}"), {}))'.format(left, op, right)
 
     def visit_Mult(self, node):
         return '__mul__'
@@ -389,7 +399,7 @@ class Pythonium(NodeVisitor):
         return '__sub__'
 
     def visit_USub(self, node):
-        return '-'
+        return '__neg__'
 
     def visit_Div(self, node):
         return '__div__'
@@ -431,19 +441,22 @@ class Pythonium(NodeVisitor):
         return '__neq__'
 
     def visit_Num(self, node):
-        return str(node.n)
+        if isinstance(node.n, float):
+            return 'pythonium_call(float, {})'.format(str(node.n))
+        else:
+            return 'pythonium_call(int, {})'.format(str(node.n))
 
     def visit_Is(self, node):
-        return '==='
+        return '__is__'
 
     def visit_Not(self, node):
-        return '!'
+        return '__neg__'
 
     def visit_IsNot(self, node):
-        return '!=='
+        return '__isnot__'
 
     def visit_UnaryOp(self, node):
-        return self.visit(node.op) + self.visit(node.operand)
+        return 'pythonium_call(pythonium_get_attribute({}, "{}"))'.format(self.visit(node.operand), self.visit(node.op))
 
     def visit_And(self, node):
         return '__and__'
@@ -465,6 +478,13 @@ class Pythonium(NodeVisitor):
                     value
                 ))
                 return
+            elif isinstance(target, Subscript):
+                self.writer.write('pythonium_call(pythonium_get_attribute({}, "__setitem__"), {}, {});'.format(
+                    self.visit(target.value),
+                    self.visit(target.slice),
+                    value,
+                ))
+                return
             else:
                 target = self.visit(target)
                 self.writer.write('{} = {};'.format(target, value))
@@ -500,12 +520,12 @@ class Pythonium(NodeVisitor):
                         else:
                             raise NotImplementedError
                     else:
-                        if isinstance(self._def_stack[-1], ClassDef):
+                        if self._def_stack and isinstance(self._def_stack[-1], ClassDefNode):
                             name = '__{}_{}'.format(self._def_stack[-1].name, target)
                         else:
                             name = target
                         self.writer.write('{} = __assignement;'.format(name))
-                if isinstance(self._def_stack[-1], ClassDef):
+                if self._def_stack and isinstance(self._def_stack[-1], ClassDefNode):
                     return target, name
 
     def visit_Expr(self, node):
@@ -528,16 +548,20 @@ class Pythonium(NodeVisitor):
         iter = reversed(ops)
         c = next(iter)
         for op in iter:
-            c = '(pythonium_get_attribute({}, {})({}))'.format(next(iter), op, c)
+            c = '(pythonium_get_attribute({}, "{}")({}))'.format(next(iter), op, c)
         return c
 
     def visit_BoolOp(self, node):
         op = self.visit(node.op)
-        return '({})'.format(op.join([self.visit(v) for v in node.values]))
+        out = self.visit(node.values[0])
+        for value in node.values[1:]:
+            v = self.visit(value)
+            out = 'pythonium_call(pythonium_get_attribute({}, "{}"), {})'.format(v, op, out)
+        return out
 
     def visit_If(self, node):
         test = self.visit(node.test)
-        self.writer.write('if({}) {{'.format(test))
+        self.writer.write('if (pythonium_is_true({})) {{'.format(test))
         self.writer.push()
         list(map(self.visit, node.body))
         self.writer.pull()
@@ -561,17 +585,18 @@ class Pythonium(NodeVisitor):
     def visit_For(self, node):
         # support only arrays
         target = node.target.id 
-        iterator_index = target + '_iterator_index'
         iterator = self.visit(node.iter) # iter is the python iterator
-        iterator_name = 'iterator_{}'.format(target)
-        self.writer.write('var {} = {};'.format(iterator_name, iterator))
-        # replace the replace target with the javascript iterator
-        self.writer.write('for (var {}=0; {} < {}.length; {}++) {{'.format(iterator_index, iterator_index, iterator_name, iterator_index))
+        self.writer.write('try {')
         self.writer.push()
-        self.writer.write('var {} = {}[{}];'.format(target, iterator_name, iterator_index))
+        self.writer.write('var __next__ = pythonium_get_attribute(iter({}), "__next__");'.format(iterator))
+        self.writer.write('while(true) {')
+        self.writer.push()
+        self.writer.write('var {} = pythonium_call(__next__);'.format(target))
         list(map(self.visit, node.body))
         self.writer.pull()
         self.writer.write('}')
+        self.writer.pull()
+        self.writer.write('} catch (x) { if (!pythonium_is_exception(x, StopIteration)) { throw x; }}')
 
     def visit_Continue(self, node):
         self.writer.write('continue;')
@@ -586,12 +611,12 @@ class Pythonium(NodeVisitor):
     def visit_ClassDef(self, node):
         # 'name', 'bases', 'keywords', 'starargs', 'kwargs', 'body', 'decorator_lis't
         if len(node.bases) == 0:
-            bases = ['object']
+            bases = ['__object']
         else:
             bases = map(self.visit, node.bases)
         bases = '[{}]'.format(', '.join(bases))
 
-        self._def_stack.append(ClassDef(node.name))
+        self._def_stack.append(ClassDefNode(node.name))
         self.writer.write('/* class definition {} */'.format(node.name))
         definitions = []
         for child in node.body:
